import { defineComponent, PropType, ref, computed, h } from 'vue'
import type { Component } from 'vue'
import { ElInput, ElForm, ElFormItem } from 'element-plus'
import type { FormInstance } from 'element-plus'
import AmountInput from './Form.vue/AmountInput.vue' // 金额输入框
import DateRangePicker from './Form.vue/DateRangePicker.vue' // 时间范围选择器
import DatePicker from './Form.vue/DatePicker.vue' // 单个日期选择器
import Select from './Form.vue/Select.vue' // 普通下拉选择框
import Switch from './Form.vue/Switch .vue'
import './Form.vue/form.scss'

import type { FormItemRule } from 'element-plus'

// 定义表单项配置接口
export interface IFormConfig {
  title?: string
  prop?: string
  label?: string
  type?: string
  btn?: JSX.Element | (() => JSX.Element)
  attrs?: Record<string, unknown>
  rules?: FormItemRule | FormItemRule[]
  render?: (
    item?: IFormConfig,
    formData?: Record<string, any>,
    index?: number,
  ) => JSX.Element
}

// 定义组件类型
type ComponentType = Component

// 支持的组件映射
const componentConfig: Record<string, ComponentType> = {
  input: ElInput,
  amountInput: AmountInput,
  dateRangePicker: DateRangePicker,
  datePicker: DatePicker,
  select: Select,
  switch: Switch,
  // 这里可以根据需要添加更多组件类型
}

export default defineComponent({
  name: 'CustomizeForm',
  props: {
    config: {
      type: Array as PropType<IFormConfig[]>,
      default: () => [],
    },
    labelWidth: {
      type: [String, Number],
      default: 100,
    },
    formItemWidth: {
      type: [String, Number],
      default: '50%',
    },
    modelValue: {
      type: Object as PropType<Record<string, unknown>>,
      default: () => ({}),
    },
    size: {
      type: String as PropType<'large' | 'default' | 'small'>,
      default: 'default',
    },
    inline: {
      type: Boolean,
      default: false,
    },
    labelPosition: {
      type: String as PropType<'left' | 'right' | 'top'>,
      default: 'right',
    },
  },
  emits: ['update:modelValue', 'validate'],
  setup(props, { emit, attrs }) {
    const formRef = ref<FormInstance>()
    const formData = ref<Record<string, unknown>>(props.modelValue)

    // 监听表单数据变化
    watch(
      () => formData.value,
      (val) => {
        emit('update:modelValue', val)
      },
      { deep: true },
    )

    // 监听外部数据变化
    watch(
      () => props.modelValue,
      (val) => {
        // console.log(84, val)

        formData.value = val
      },
      { immediate: true, deep: true },
    )

    // 获取form表单的属性
    const getFormAttrs = computed(() => ({
      ...attrs,
      style: { width: '100%' },
    }))

    // 获取组件项的属性
    const getComponentAttrs = (item: IFormConfig) => {
      return item.attrs ? { ...item.attrs } : {}
    }

    // 表单验证方法
    const validate = async () => {
      if (!formRef.value) return false
      return await formRef.value.validate()
    }

    // 重置表单
    const resetFields = async () => {
      await formRef.value?.resetFields()
    }

    return {
      formRef,
      formData,
      validate,
      resetFields,
      getFormAttrs,
      getComponentAttrs,
    }
  },
  render() {
    return (
      <ElForm
        ref="formRef"
        model={this.formData}
        labelWidth={this.labelWidth}
        size={this.size}
        inline={this.inline}
        labelPosition={this.labelPosition}
        {...this.getFormAttrs}
        style={{ display: 'flex', flexWrap: 'wrap' }}
        class="customForm"
      >
        {this.config.map((item, index) =>
          item.title ? (
            <div
              style={{
                display: 'flex',
                width: '100%',
                flexWrap: 'wrap',
              }}
            >
              <div
                style={{
                  width: '100%',
                  textAlign: 'center',
                  fontWeight: 700,
                  marginBottom: '20px',
                  fontSize: '16px',
                }}
              >
                {item.title}
              </div>
              {item.render && item.render(item, this.formData, this)}
            </div>
          ) : (
            <ElFormItem
              key={index}
              prop={item.prop}
              label={item.label}
              style={{
                width: item.attrs?.width || this.formItemWidth || '50%',
              }}
              rules={item.rules}
            >
              {item.render
                ? item.render(item)
                : h(componentConfig[item.type] || ElInput, {
                    modelValue: this.formData[item.prop],
                    'onUpdate:modelValue': (value: unknown) => {
                      this.formData[item.prop] = value
                    },
                    ...this.getComponentAttrs(item),
                  })}
              {item.btn && item.btn()}
            </ElFormItem>
          ),
        )}
      </ElForm>
    )
  },
})
